﻿using DomainModel.Entities;
using NHibernate;
using System.Collections.Generic;
using System;
using NHibernate.Criterion;
using System.Linq;

namespace DAL
{
    public class NHibernateSample
    {
        private ISession _session;

        public NHibernateSample(ISession session)
        {
            _session = session;
        }

        public ISession Session
        {
            set { _session = value; }
        }

        public Customer GetCustomerById(int customerId)
        {
            return _session.Get<Customer>(customerId);
        }

        public int CreateCustomerTransaction(Customer customer)
        {
            using (ITransaction tx = _session.BeginTransaction())
            {
                try
                {
                    int newId = (int)_session.Save(customer);
                    _session.Flush();
                    tx.Commit();
                    return newId;
                }
                catch (HibernateException)
                {
                    tx.Rollback();
                    throw;
                }
            }
        }

        public void DeleteCustomerTransaction(Customer customer)
        {
            using (ITransaction tx = _session.BeginTransaction())
            {
                try
                {
                    _session.Delete(customer);
                    _session.Flush();
                    tx.Commit();
                }
                catch (HibernateException)
                {
                    tx.Rollback();
                    throw;
                }
            }
        }

        public void UpdateCustomerTransaction(Customer customer)
        {
            using (ITransaction tx = _session.BeginTransaction())
            {
                try
                {
                    _session.Update(customer);
                    _session.Flush();
                    tx.Commit();
                }
                catch (HibernateException)
                {
                    tx.Rollback();
                    throw;
                }
            }
        }

        public void SaveOrUpdateCustomersTransaction(IList<Customer> customers)
        {
            using (ITransaction tx = _session.BeginTransaction())
            {
                try
                {
                    foreach (Customer c in customers)
                        _session.SaveOrUpdate(c);
                    _session.Flush();
                    tx.Commit();
                }
                catch (HibernateException)
                {
                    tx.Rollback();
                    throw;
                }
            }
        }

        public IList<Customer> ReturnFullName(string firstname, string lastname)
        {
            return _session
                 .CreateQuery("select from Customer c where c.Name.Firstname=:fn and c.Name.Lastname=:ln")
                .SetString("fn", firstname)
                .SetString("ln", lastname)
                .List<Customer>();
        }

        #region 一对多关联查询
        //1.原生SQL关联查询
        public IList<Customer> UseSQL_GetCustomersWithOrders(DateTime orderDate)
        {
            return _session.CreateSQLQuery("select distinct {customer.*} from Customer {customer}" +
            " inner join [Order] o on o.Customer={customer}.CustomerId where o.OrderDate> :orderDate")
                .AddEntity("customer", typeof(Customer))
                .SetDateTime("orderDate", orderDate)
                .List<Customer>();
        }

        //2.HQL关联查询 这里用 NHibernate 2.1.0版本是不可以通过的，报如下错误：
        //failed: NHibernate.Hql.Ast.ANTLR.QuerySyntaxException : c.Orders is not mapped [select distinct c from DomainModel.Entities.Customer c ,c.Orders o where o.OrderDate > :orderDate]
        public IList<Customer> UseHQL_GetCustomersWithOrders(DateTime orderDate)
        {
            return _session.CreateQuery("select distinct c from Customer c ,"
                + "c.Orders.elements o where o.OrderDate > :orderDate")
                .SetDateTime("orderDate", orderDate)
                .List<Customer>();
        }

        //3. Criteria API 关连查询
        public IList<Customer> UseCriteriaAPI_GetCustomersWithOrders(DateTime orderDate)
        {
            return _session.CreateCriteria(typeof(Customer))
                .CreateCriteria("Orders")
                .Add(Restrictions.Gt("OrderDate", orderDate))
                .SetResultTransformer(new NHibernate.Transform.DistinctRootEntityResultTransformer())
                //或者.SetResultTransformer(NHibernate.CriteriaUtil.DistinctRootEntity)
                .List<Customer>();
        }

        //投影
        public IList<Customer> UseCriteriaAPI_GetDistinctCustomers(DateTime orderDate)
        {
            IList<int> ids = _session.CreateCriteria(typeof(Customer))
                .SetProjection(Projections.Distinct(Projections.ProjectionList()
                                                                  .Add(Projections.Property("CustomerId"))
                                                              )
                                   )
                .CreateCriteria("Orders")
                .Add(Restrictions.Gt("OrderDate", orderDate))
                .List<int>();

            return _session.CreateCriteria(typeof(Customer))
                .Add(Restrictions.In("CustomerId", ids.ToArray<int>()))
                .List<Customer>();
        }
        #endregion

        #region 多对多关联查询
        //原生SQL查询
        public IList<Customer> UseSQL_GetCustomersWithOrdersHavingProduct(DateTime orderDate)
        {
            return _session.CreateSQLQuery("select distinct {customer.*} from Customer {customer}" +
            " inner join [Order] o on o.Customer={customer}.CustomerId" +
            " inner join OrderProduct op on o.OrderId=op.[Order]" +
            " inner join Product p on op.Product=p.ProductId where o.OrderDate> :orderDate")
                .AddEntity("customer", typeof(Customer))
                .SetDateTime("orderDate", orderDate)
                .List<Customer>();
        }

        //HQL 关联查询
        public IList<Customer> UseHQL_GetCustomersWithOrdersHavingProduct(DateTime orderDate)
        {
            return _session.CreateQuery("select distinct c from Customer c ,"
                + " c.Orders.elements o where o.OrderDate > :orderDate")
                .SetDateTime("orderDate", orderDate)
                .List<Customer>();
        }

        //Criteria API关联查询
        public IList<Customer> UseCriteriaAPI_GetCustomerswithOrdersHavingProduct(string userName, string productName, DateTime beginOrderDate)
        {
            return _session.CreateCriteria(typeof(Customer))
                .Add(Restrictions.Eq("Name.Firstname", userName))
                .CreateCriteria("Orders")
                .Add(Restrictions.Gt("OrderDate", beginOrderDate))
                .CreateCriteria("Products")
                .Add(Restrictions.Eq("Name", productName))
                .List<Customer>();
        }

        //Criteria API关联查询
        public IList<Customer> UseCriteriaAPI_GetCustomerswithOrdersHavingProduct(int productId)
        {
            return _session.CreateCriteria(typeof(Customer))
                .CreateCriteria("Orders")
                .CreateCriteria("Products")
                .Add(Restrictions.Eq("ProductId", productId))
                .List<Customer>();
        }

        #endregion
    }
}