﻿using DomainModel.Entities;
using NHibernate;
using NUnit.Framework;
using System;
using System.Collections.Generic;
using System.Linq;

namespace DAL.Test
{
    [TestFixture]
    public class NHibernateSampleFixture
    {
        #region Setup/Teardown

        [SetUp]
        public void Setup()
        {
            _session = _helper.GetSession();
            _sample = new NHibernateSample(_session);
        }

        #endregion

        private ISession _session;
        private SessionManager _helper;
        private NHibernateSample _sample;

        [TestFixtureSetUp]
        public void TestFixtureSetup()
        {
            _helper = new SessionManager();
        }

        //Get查询测试

        [Test]
        public void GetCustomerByIdTest()
        {
            Customer customer = _sample.GetCustomerById(1);
            int customerId = customer.CustomerId;
            Assert.AreEqual(1, customerId);
        }

        //事务测试

        [Test]
        public void CreateCustomerTransactionTest()
        {
            var customer = new Customer() { Name = new Name() { Firstname = "YJing", Lastname = "Lee" } };
            int newIdentity = _sample.CreateCustomerTransaction(customer);
            var testCustomer = _sample.GetCustomerById(newIdentity);
            Assert.IsNotNull(testCustomer);
        }

        [Test]
        [ExpectedException(typeof(NHibernate.HibernateException))]
        public void CreateCustomerThrowExceptionOnFailTest()
        {
            var customer = new Customer()
            {
                Name = new Name()
                         {
                             Firstname = "012345678901234567890123456789012345678901234567890123456789",
                             Lastname = "YJingLee"
                         }
            };
            _sample.CreateCustomerTransaction(customer);
        }

        //并发测试
        [Test]
        public void UpdateConcurrencyViolationCanotThrowException()
        {
            Customer c1 = _sample.GetCustomerById(1);
            Customer c2 = _sample.GetCustomerById(1);
            c1.Name.Firstname = "Cnblog";
            c2.Name.Firstname = "Kaibo";

            _sample.UpdateCustomerTransaction(c1);
            _sample.UpdateCustomerTransaction(c2);
        }

        [Test]
        [ExpectedException(typeof(NHibernate.StaleObjectStateException))]
        public void DeleteConcurrencyViolationCanotThrowException()
        {
            Customer c1 = _sample.GetCustomerById(2);
            Customer c2 = _sample.GetCustomerById(2);

            _sample.DeleteCustomerTransaction(c1);
            _sample.DeleteCustomerTransaction(c2);
        }

        // 测试组件
        [Test]
        public void ReturnFullNameTest()
        {
            IList<Customer> customers = _sample.ReturnFullName("Kaibo", "Hao");
            foreach (Customer c in customers)
            {
                Assert.AreEqual("Kaibo Hao", c.Name.Fullname);
            }
        }

        // 测试一对多关联查询
        [Test]
        public void UseHQL_GetCustomersWithOrdersTest()
        {
            //SQL关系查询
            //IList<Customer> customers = _sample.UseSQL_GetCustomersWithOrders(new DateTime(2008, 10, 1));

            //HQL关系查询
            //IList<Customer> customers = _sample.UseHQL_GetCustomersWithOrders(new DateTime(2008, 10, 1));

            //Criteria 关联查询
            IList<Customer> customers = _sample.UseCriteriaAPI_GetCustomersWithOrders(new DateTime(2008, 10, 1));

            //投影 关联查询
            //IList<Customer> customers = _sample.UseCriteriaAPI_GetDistinctCustomers(new DateTime(2008, 10, 1));
            foreach (Customer c in customers)
            {
                foreach (Order o in c.Orders)
                {
                    Assert.GreaterOrEqual(o.OrderDate, new DateTime(2008, 10, 1));
                }
            }
            foreach (Customer c in customers)
            {
                Assert.AreEqual(1, customers.Count<Customer>(x => x == c));
            }

        }

        // 测试多对多关联查询
        [Test]
        public void UseCriteriaAPI_GetCustomerswithOrdersHavingProductTest()
        {
            IList<Customer> customers = _sample.UseCriteriaAPI_GetCustomerswithOrdersHavingProduct("Kaibo", "Cnblogs", new DateTime(2008, 10, 1));
            bool found = false;
            foreach (Customer c in customers)
            {
                foreach (Order o in c.Orders)
                {
                    foreach (Product p in o.Products)
                    {
                        if (p.Name == "Cnblogs")
                        {
                            found = true;
                            break;
                        }
                    }
                }
            }
            Assert.IsTrue(found);
        }
    }
}